import { Direction } from "@minecraft/server";
import { Vector3 } from "../_lib/vector";
export class Common {
    static getAdjacentBlock(face, dimension, location) {
        let offset = Vector3.ZERO;
        switch (face) {
            case Direction.North:
                offset = new Vector3(0, 0, -1);
                break;
            case Direction.South:
                offset = new Vector3(0, 0, 1);
                break;
            case Direction.East:
                offset = new Vector3(1, 0, 0);
                break;
            case Direction.West:
                offset = new Vector3(-1, 0, 0);
                break;
            case Direction.Up:
                offset = new Vector3(0, 1, 0);
                break;
            case Direction.Down:
                offset = new Vector3(0, -1, 0);
                break;
        }
        return dimension.getBlock(Vector3.add(location, offset));
    }
    static getBlockNeighbors(block) {
        const directions = ['north', 'south', 'east', 'west', 'above', 'below'];
        const invalidTypes = ['minecraft:short_grass', 'minecraft:deadbush'];
        const invalidTags = ['plant', 'leaves'];
        const invalidCoreTypes = ['leaves'];
        const neighbors = directions.reduce((acc, direction) => {
            const neighbor = block[direction]();
            if (neighbor && !invalidTypes.includes(neighbor.typeId) && !invalidTags.some(tag => neighbor.hasTag(tag)) && !neighbor.isAir && !neighbor.isLiquid && !invalidCoreTypes.some(type => neighbor.typeId.includes(type))) {
                acc[direction] = neighbor;
            }
            return acc;
        }, {});
        return neighbors;
    }
    static getNeighborIterator(neighbors) {
        const blocks = [];
        if (neighbors.north)
            blocks.push(neighbors.north);
        if (neighbors.south)
            blocks.push(neighbors.south);
        if (neighbors.east)
            blocks.push(neighbors.east);
        if (neighbors.west)
            blocks.push(neighbors.west);
        if (neighbors.above)
            blocks.push(neighbors.above);
        if (neighbors.below)
            blocks.push(neighbors.below);
        return blocks;
    }
}
