import { world } from "@minecraft/server";
import { Vector3 } from "../_lib/vector";
export class Leaf {
    constructor() {
        this.onTick = this.onTick.bind(this);
    }
    onTick(event) {
        if (event.block.permutation.getState('bf:decay') === false) {
            return;
        }
        if (!this.hasNeighbor(event.block)) {
            event.dimension.runCommand(`setblock ${event.block.x} ${event.block.y} ${event.block.z} air destroy`);
        }
    }
    hasNeighbor(block) {
        const directions = [block.above, block.below, block.north, block.south, block.east, block.west];
        return directions.some(direction => {
            const neighbor = direction.call(block);
            let woodCheck = false;
            if (neighbor && neighbor.hasTag('wood')) {
                woodCheck = true;
            }
            else if (neighbor && neighbor.typeId === block.typeId) {
                const dimension = block.dimension;
                const blockPos = block.location;
                for (let x = -3; x <= 3; x++) {
                    for (let y = -3; y <= 3; y++) {
                        for (let z = -3; z <= 3; z++) {
                            const checkPos = new Vector3(blockPos.x + x, blockPos.y + y, blockPos.z + z);
                            const checkBlock = dimension.getBlock(checkPos);
                            if (checkBlock && checkBlock.hasTag('wood')) {
                                woodCheck = true;
                                break;
                            }
                        }
                        if (woodCheck)
                            break;
                    }
                    if (woodCheck)
                        break;
                }
            }
            return neighbor && !neighbor.isAir && !neighbor.isLiquid && woodCheck;
        });
    }
}
Leaf.id = 'bf_rb:leaf';
world.afterEvents.playerPlaceBlock.subscribe((event) => {
    if (event.block.hasTag('vxl_custom') && event.block.hasTag('leaves')) {
        event.block.setPermutation(event.block.permutation.withState('bf:decay', false));
    }
});
