import { EquipmentSlot, GameMode, world } from "@minecraft/server";
import { Inventory } from "../_lib/inventory";
import { Vector3 } from "../_lib/vector";
export class SaplingComponent {
    constructor() {
        this.onRandomTick = this.onRandomTick.bind(this);
        this.onPlayerInteract = this.onPlayerInteract.bind(this);
    }
    onRandomTick(event) {
        this.processGrowth(event.block, 9);
    }
    onPlayerInteract(event) {
        if (!event.player)
            return;
        const inventory = new Inventory(event.player);
        const i = inventory.getEquippableComponent();
        const itemStack = i.getEquipment(EquipmentSlot.Mainhand);
        if (itemStack && itemStack.typeId === 'minecraft:bone_meal') {
            let updatedItemStack = undefined;
            if (event.player.getGameMode() === GameMode.creative) {
                updatedItemStack = itemStack;
            }
            else if (itemStack.amount > 1) {
                itemStack.amount--;
                updatedItemStack = itemStack;
            }
            this.processGrowth(event.block, 9);
            i.setEquipment(EquipmentSlot.Mainhand, updatedItemStack);
        }
        event.block.dimension.spawnParticle('minecraft:crop_growth_emitter', Vector3.add(event.block, 0.5));
    }
    processGrowth(block, maxValue) {
        let growth_bit = block.permutation.getState('bf:growth_bit');
        if (growth_bit < maxValue) {
            block.setPermutation(block.permutation.withState('bf:growth_bit', growth_bit + 1));
        }
        else if (growth_bit === 9) {
            const structure = world.structureManager.get('vxl_vbrm/custom_tree');
            if (!structure)
                throw Error(`finding structure vxl_vbrm/custom_tree`);
            //Check if tree strucrure is obstructed
            const raycastHit = block.dimension.getBlockFromRay(block, new Vector3(0, 1, 0), { maxDistance: structure.size.y - 1 });
            if (raycastHit && !raycastHit.block.isAir && !block.hasTag('leaves')) {
                return;
            }
            const location = new Vector3(block.x - Math.floor(structure.size.x / 2), block.y, block.z - Math.floor(structure.size.z / 2));
            world.structureManager.place(structure, block.dimension, location);
        }
    }
}
SaplingComponent.id = 'bf_rb:sapling';
