import { BlockPermutation, EquipmentSlot, GameMode, system, world } from "@minecraft/server";
import { Common } from "./common";
import { slabTypeIds, SoundDefinition } from "../global";
import { Inventory } from "../_lib/inventory";
export class SlabHandler {
    static onReady() {
        if (!SlabHandler.initialized) {
            world.afterEvents.itemUseOn.subscribe((event) => {
                if (!slabTypeIds.includes(event.itemStack.typeId))
                    return;
                SlabHandler.onSlabUsed(event);
                SlabHandler.interactionCache.add(event.source.id);
                system.waitTicks(2).then(() => SlabHandler.interactionCache.delete(event.source.id));
            });
            world.beforeEvents.playerInteractWithBlock.subscribe((event) => {
                if (!event.itemStack)
                    return;
                if (!slabTypeIds.includes(event.itemStack.typeId))
                    return;
                system.waitTicks(1).then(() => {
                    if (!SlabHandler.interactionCache.has(event.player.id)) {
                        SlabHandler.interactionCache.add(event.player.id);
                        this.setNextSlab(event.player, event.block, event.itemStack);
                        system.waitTicks(1).then(() => SlabHandler.interactionCache.delete(event.player.id));
                    }
                });
            });
            SlabHandler.initialized = true;
        }
        else
            throw Error('Instance of SlabHandler already active.');
    }
    static onSlabUsed(event) {
        const block = event.block;
        const faceY = event.faceLocation.y;
        const placedBlock = Common.getAdjacentBlock(event.blockFace, event.block.dimension, event.block.location);
        if (!placedBlock)
            return;
        if ((block.typeId === event.itemStack.typeId) && SlabHandler.setAsDoubleStack(block, faceY)) {
            placedBlock.setPermutation(BlockPermutation.resolve('minecraft:air'));
            return;
        }
    }
    static setAsDoubleStack(block, faceY) {
        const state = block.permutation.getState(SlabHandler.stateId);
        if (state && (state === 'top' && faceY < 0.45) || (state === 'bottom' && faceY === 0.5)) {
            block.setPermutation(block.permutation.withState(SlabHandler.fullBitStateId, true));
            return true;
        }
        return false;
    }
    static setNextSlab(player, block, itemStack) {
        let nextBlock = block.above();
        if (nextBlock?.typeId !== itemStack.typeId)
            nextBlock = block.below();
        if (nextBlock?.typeId === itemStack.typeId) {
            nextBlock.setPermutation(nextBlock.permutation.withState(SlabHandler.fullBitStateId, true));
            let updatedItemStack = undefined;
            if (player.getGameMode() === GameMode.creative) {
                updatedItemStack = itemStack;
            }
            else if (itemStack.amount > 1) {
                itemStack.amount--;
                updatedItemStack = itemStack;
            }
            const inventory = new Inventory(player);
            const i = inventory.getEquippableComponent();
            i.setEquipment(EquipmentSlot.Mainhand, updatedItemStack);
            block.dimension.playSound(SoundDefinition.PlaceWood, block);
        }
    }
}
SlabHandler.id = 'bf_rb:slab';
SlabHandler.initialized = false;
SlabHandler.stateId = 'minecraft:vertical_half';
SlabHandler.fullBitStateId = 'bf:full_bit';
SlabHandler.interactionCache = new Set();
