import { system, world } from "@minecraft/server";
import { strippableBlockTypeIds } from "../global";
export class StripHandler {
    static onReady() {
        if (!StripHandler.initialized) {
            world.beforeEvents.playerInteractWithBlock.subscribe((event) => {
                if (!event.player || !event.itemStack || !strippableBlockTypeIds.includes(event.block.typeId) || StripHandler.interactionCache.has(event.player.id))
                    return;
                const itemStack = event.itemStack;
                StripHandler.interactionCache.add(event.player.id);
                const run = system.run(() => {
                    StripHandler.onPlayerInteract(event.player, event.block, itemStack);
                    system.clearRun(run);
                });
            });
        }
        else
            throw Error('Instance of StripHandler already active.');
    }
    static onPlayerInteract(player, block, itemStack) {
        if (!itemStack || !itemStack.typeId.includes('_axe'))
            return;
        const split = block.typeId.split(':');
        const blockFace = block.permutation.getState('minecraft:block_face');
        let typeId = `${split[0]}:stripped_${split[1]}`;
        if (block.typeId.includes('end_log'))
            typeId = 'bf_rb:stripped_end_log';
        else if (block.typeId.includes('end_wood'))
            typeId = 'bf_rb:stripped_end_wood';
        try {
            block.setType(typeId);
        }
        catch (error) {
            throw Error(`${error}`);
        }
        ;
        block.setPermutation(block.permutation.withState('minecraft:block_face', blockFace));
        StripHandler.interactionCache.delete(player.id);
    }
}
StripHandler.initialized = false;
StripHandler.interactionCache = new Set();
