import { BlockPermutation, EntityEquippableComponent, EntityInventoryComponent, EquipmentSlot, GameMode, ItemStack, system, world } from "@minecraft/server";
import { DoorComponent } from "./components/door";
import { FenceComponent } from "./components/fence";
import { FenceGateComponent } from "./components/fence_gate";
import { SlabHandler } from "./components/slab";
import { StairsComponent } from "./components/stairs";
import { StripHandler } from "./components/strippable";
import { TrapdoorComponent } from "./components/trapdoor";
import { Inventory } from "./_lib/inventory";
import { FallenLogComponent } from "./components/fallen_log";
import { OnDestoryBerryComponent, OnDestoryShearFallenBrancesComponent, OnDestoryShearFallenLeavesComponent, OnDestoryShearGrassComponent, OnDestoryShearPlantsComponent, OnPlayerDestoryUngrownComponent } from "./components/destory_components";
import { OnGrowBerryComponent, OnHarvestComponent, OnInteractFallingLeavesComponent, OnInteractGrowComponent } from "./components/interaction_components";
import { OnTickGrowBerryComponent, OnTickGrowComponent, OnTickGrowVanillaComponent } from "./components/tick_components";
import { growPlants } from "./util/growPlants";
SlabHandler.onReady();
StripHandler.onReady();
world.beforeEvents.worldInitialize.subscribe(initEvent => {
    initEvent.blockComponentRegistry.registerCustomComponent('bf_rb:door', new DoorComponent());
    initEvent.blockComponentRegistry.registerCustomComponent('bf_rb:fence_gate', new FenceGateComponent());
    initEvent.blockComponentRegistry.registerCustomComponent('bf_rb:fence', new FenceComponent());
    initEvent.blockComponentRegistry.registerCustomComponent('bf_rb:stairs', new StairsComponent());
    initEvent.blockComponentRegistry.registerCustomComponent('bf_rb:trapdoor', new TrapdoorComponent());
    initEvent.blockComponentRegistry.registerCustomComponent('bf_rb:on_player_placed_fallen_log', new FallenLogComponent());
    initEvent.blockComponentRegistry.registerCustomComponent(OnDestoryShearGrassComponent.id, new OnDestoryShearGrassComponent());
    initEvent.blockComponentRegistry.registerCustomComponent(OnDestoryShearFallenLeavesComponent.id, new OnDestoryShearFallenLeavesComponent());
    initEvent.blockComponentRegistry.registerCustomComponent(OnDestoryShearFallenBrancesComponent.id, new OnDestoryShearFallenBrancesComponent());
    initEvent.blockComponentRegistry.registerCustomComponent(OnInteractGrowComponent.id, new OnInteractGrowComponent());
    initEvent.blockComponentRegistry.registerCustomComponent(OnInteractFallingLeavesComponent.id, new OnInteractFallingLeavesComponent());
    initEvent.blockComponentRegistry.registerCustomComponent(OnTickGrowComponent.id, new OnTickGrowComponent());
    initEvent.blockComponentRegistry.registerCustomComponent(OnDestoryShearPlantsComponent.id, new OnDestoryShearPlantsComponent());
    initEvent.blockComponentRegistry.registerCustomComponent(OnTickGrowVanillaComponent.id, new OnTickGrowVanillaComponent());
    initEvent.blockComponentRegistry.registerCustomComponent(OnTickGrowBerryComponent.id, new OnTickGrowBerryComponent());
    initEvent.blockComponentRegistry.registerCustomComponent(OnGrowBerryComponent.id, new OnGrowBerryComponent());
    initEvent.blockComponentRegistry.registerCustomComponent(OnPlayerDestoryUngrownComponent.id, new OnPlayerDestoryUngrownComponent());
    initEvent.blockComponentRegistry.registerCustomComponent(OnHarvestComponent.id, new OnHarvestComponent());
    initEvent.blockComponentRegistry.registerCustomComponent(OnDestoryBerryComponent.id, new OnDestoryBerryComponent());
});
function _itemReplacer(player) {
    if (player.getGameMode() !== GameMode.creative)
        return;
    const inventoryComponent = player.getComponent(EntityInventoryComponent.componentId);
    const container = inventoryComponent.container;
    for (let i = 0; i < container.size; i++) {
        const itemStack = container.getItem(i);
    }
}
system.afterEvents.scriptEventReceive.subscribe((event) => {
    if (!event.sourceEntity)
        return;
    if (event.id !== 'bf_rb:banned_block_dectector')
        return;
    const player = event.sourceEntity;
    const container = new Inventory(player).container;
    for (let i = 0; i < container.size; i++) {
        const itemStack = container.getItem(i);
        if (itemStack && itemStack.typeId === event.message) {
            const newItemStack = new ItemStack(`${itemStack.typeId.replace('_tile', '')}`, itemStack.amount);
            container.setItem(i, newItemStack);
            break;
        }
    }
});
world.beforeEvents.playerInteractWithBlock.subscribe((event) => {
    if (event.isFirstEvent) {
        if (event.itemStack && event.itemStack.typeId == "minecraft:bone_meal" &&
            (event.block.typeId == "bf_rb:sandy_mud" ||
                event.block.typeId == "bf_rb:redwood_podzol" ||
                event.block.typeId == "bf_rb:yellow_grass" ||
                event.block.typeId == "bf_rb:wisteria_grass" ||
                event.block.typeId == "bf_rb:red_grass" ||
                event.block.typeId == "bf_rb:pine_grass" ||
                event.block.typeId == "bf_rb:magnolia_grass" ||
                event.block.typeId == "bf_rb:dry_soil" ||
                event.block.typeId == "bf_rb:snowy_pine_grass" ||
                event.block.typeId == "bf_rb:purple_growth_end_stone" ||
                event.block.typeId == "bf_rb:yellow_growth_end_stone")) {
            growPlants(event);
        }
    }
});
world.beforeEvents.playerBreakBlock.subscribe((event) => {
    if (event.block.typeId.includes('bf_rb') && event.block.typeId.includes('leaves')) {
        const equipment = event.player.getComponent(EntityEquippableComponent.componentId);
        const itemStack = equipment.getEquipment(EquipmentSlot.Mainhand);
        const location = event.block.location;
        const leafItem = new ItemStack(event.block.typeId);
        const stickItem = new ItemStack('minecraft:stick');
        const instance = system.run(() => {
            if (itemStack && itemStack.typeId === 'minecraft:shears') {
                event.dimension.spawnItem(leafItem, location);
                if (Math.random() > 0.5) {
                    const amount = Math.floor(Math.random() * 2) + 1;
                    for (let i = 0; i < amount; i++) {
                        event.dimension.spawnItem(stickItem, location);
                    }
                }
            }
            system.clearRun(instance);
        });
    }
});
const grassBlocks = [
    'bf_rb:lush_grass',
    'bf_rb:pine_grass',
    'bf_rb:snowy_pine_grass',
    'bf_rb:wisteria_grass',
    'bf_rb:magnolia_grass',
    'bf_rb:yellow_grass',
    'bf_rb:ice_grass_block',
    'bf_rb:red_grass'
];
world.beforeEvents.playerInteractWithBlock.subscribe((event) => {
    if (!event.isFirstEvent || !event.itemStack ||
        !grassBlocks.includes(event.block.typeId))
        return;
    let blockType;
    if (event.itemStack.typeId.includes('_hoe')) {
        blockType = 'minecraft:farmland';
    }
    else if (event.itemStack.typeId.includes('_shovel')) {
        blockType = 'minecraft:grass_path';
    }
    else
        return;
    const run = system.run(() => {
        event.block.setPermutation(BlockPermutation.resolve(blockType));
        event.player.playSound('dig.grass');
        event.player.playAnimation('animation.bf_rb.player_swing');
        system.clearRun(run);
    });
});
